VERTEX_START

precision highp float;
attribute vec3 Position;
attribute vec2 TexCoord0;
varying mediump vec2 Out_Tex0;
void main() {
  gl_Position = vec4(Position.xy, 0.0, 1.0);
  Out_Tex0 = vec2(TexCoord0.x, TexCoord0.y);
}

VERTEX_END

FRAGMENT_START

precision mediump float;
uniform sampler2D Diffuse;
varying mediump vec2 Out_Tex0;
uniform mediump vec4 u_settings;
uniform mediump float u_settings1;

vec3 rgbToHsl(vec3 c) {
    float maxC = max(max(c.r, c.g), c.b);
    float minC = min(min(c.r, c.g), c.b);
    float delta = maxC - minC;
    float h = 0.0;
    float s = 0.0;
    float l = (maxC + minC) / 2.0;

    if (delta != 0.0) {
        if (l < 0.5) {
            s = delta / (maxC + minC);
        } else {
            s = delta / (2.0 - maxC - minC);
        }

        if (maxC == c.r) {
            h = (c.g - c.b) / delta;
        } else if (maxC == c.g) {
            h = 2.0 + (c.b - c.r) / delta;
        } else {
            h = 4.0 + (c.r - c.g) / delta;
        }

        h = mod(h / 6.0, 1.0);
    }

    return vec3(h, s, l);
}

float hueToRgb(float p, float q, float t) {
    if (t < 1.0 / 6.0) return p + (q - p) * 6.0 * t;
    if (t < 1.0 / 2.0) return q;
    if (t < 2.0 / 3.0) return p + (q - p) * (2.0 / 3.0 - t) * 6.0;
    return p;
}

vec3 hslToRgb(vec3 c) {
    float h = c.x;
    float s = c.y;
    float l = c.z;

    float r, g, b;

    if (s == 0.0) {
        r = g = b = l; // achromatic
    } else {
        float q = (l < 0.5) ? (l * (1.0 + s)) : (l + s - l * s);
        float p = 2.0 * l - q;
        float rH = mod(h + 1.0 / 3.0, 1.0);
        float gH = mod(h, 1.0);
        float bH = mod(h - 1.0 / 3.0, 1.0);
        r = hueToRgb(p, q, rH);
        g = hueToRgb(p, q, gH);
        b = hueToRgb(p, q, bH);
    }

    return vec3(r, g, b);
}

void main() {
    vec4 color = texture2D(Diffuse, Out_Tex0);
    
    const vec3 fLuminance = vec3(0.299, 0.587, 0.114);
    color.rgb += u_settings.x; // Brightness
    float gray = dot(color.rgb, fLuminance);
    color.rgb = mix(vec3(gray), color.rgb, u_settings.y); // Saturation
    color.rgb = ((color.rgb - 0.5) * max(u_settings.z, 0.0)) + 0.5; // Contrast
    color.rgb = pow(color.rgb, vec3(1.0 / u_settings.w)); // Gamma

    // Apply hue
    if (u_settings1 != 0.0) {
        vec3 hsl = rgbToHsl(color.rgb);
        hsl.x += u_settings1;
        hsl.x = mod(hsl.x, 1.0);
        color.rgb = hslToRgb(hsl);
    }
   
    gl_FragColor.rgb = color.rgb;
    gl_FragColor.a = 1.0;
}

FRAGMENT_END