VERTEX_START

precision highp float;
attribute vec3 Position;
attribute vec2 TexCoord0;
varying mediump vec2 Out_Tex0;
void main() {
  gl_Position = vec4(Position.xy, 0.0, 1.0);
  Out_Tex0 = vec2(TexCoord0.x, TexCoord0.y);
}

VERTEX_END

FRAGMENT_START

precision mediump float;
uniform sampler2D Diffuse;
varying mediump vec2 Out_Tex0;
uniform mediump vec3 u_texelDelta;
uniform mediump vec3 u_settings;

#define FXAA_SPAN_MAX u_settings.x
#define FXAA_REDUCE_MUL u_settings.y
#define FXAA_REDUCE_MIN u_settings.z

// https://github.com/hrydgard/ppsspp/blob/master/assets/shaders/fxaa.fsh
vec4 fxaa_texture2D(sampler2D Diffuse, vec2 Out_Tex0) {
    vec3 rgbNW = texture2D(Diffuse, Out_Tex0 + (vec2(-1.0, -1.0) * u_texelDelta.xy)).xyz;
    vec3 rgbNE = texture2D(Diffuse, Out_Tex0 + (vec2(1.0, -1.0) * u_texelDelta.xy)).xyz;
    vec3 rgbSW = texture2D(Diffuse, Out_Tex0 + (vec2(-1.0, 1.0) * u_texelDelta.xy)).xyz;
    vec3 rgbSE = texture2D(Diffuse, Out_Tex0 + (vec2(1.0, 1.0) * u_texelDelta.xy)).xyz;
    vec3 rgbM = texture2D(Diffuse, Out_Tex0).xyz;

    vec3 luma = vec3(0.299, 0.587, 0.114);
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM = dot(rgbM, luma);

    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y = ((lumaNW + lumaSW) - (lumaNE + lumaSE));

    float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL), FXAA_REDUCE_MIN);

    float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);

    dir = min(vec2(FXAA_SPAN_MAX, FXAA_SPAN_MAX), max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX), dir * rcpDirMin)) * u_texelDelta.xy;

    vec3 rgbA = (1.0 / 2.0) * (
        texture2D(Diffuse, Out_Tex0 + dir * (1.0 / 3.0 - 0.5)).xyz +
        texture2D(Diffuse, Out_Tex0 + dir * (2.0 / 3.0 - 0.5)).xyz
    );
    vec3 rgbB = rgbA * (1.0 / 2.0) + (1.0 / 4.0) * (
        texture2D(Diffuse, Out_Tex0 + dir * (0.0 / 3.0 - 0.5)).xyz +
        texture2D(Diffuse, Out_Tex0 + dir * (3.0 / 3.0 - 0.5)).xyz
    );
    float lumaB = dot(rgbB, luma);

    vec3 result;
    if ((lumaB < lumaMin) || (lumaB > lumaMax)) {
        result = rgbA;
    } else {
        result = rgbB;
    }

    return vec4(result, 1.0);
}

void main() {
    vec4 color = fxaa_texture2D(Diffuse, Out_Tex0);
    gl_FragColor.rgb = color.rgb;
    gl_FragColor.a = 1.0;
}

FRAGMENT_END